@@ -73,6 +73,13 @@ public class Functions {
             Geometry geom = (Geometry) o;
             return org.apache.sedona.common.Functions.buffer(geom, radius);
         }
+
+        @DataTypeHint(value = "RAW", bridgedTo = org.locationtech.jts.geom.Geometry.class)
+        public Geometry eval(@DataTypeHint(value = "RAW", bridgedTo = org.locationtech.jts.geom.Geometry.class)
+                             Object o, @DataTypeHint("Double") Double radius, @DataTypeHint("String") String params) {
+            Geometry geom = (Geometry) o;
+            return org.apache.sedona.common.Functions.buffer(geom, radius, params);
+        }
     }
 
     public static class ST_ClosestPoint extends ScalarFunction {
@@ -290,6 +297,15 @@ public class Functions {
         }
     }
 
+    public static class ST_LineLocatePoint extends ScalarFunction {
+        @DataTypeHint("Double")
+        public Double eval(@DataTypeHint(value = "RAW", bridgedTo = org.locationtech.jts.geom.Geometry.class) Object o, @DataTypeHint(value = "RAW", bridgedTo = org.locationtech.jts.geom.Geometry.class) Object p){
+            Geometry geom = (Geometry) o;
+            Geometry point = (Geometry) p;
+            return org.apache.sedona.common.Functions.lineLocatePoint(geom, point);
+        }
+    }
+
     public static class ST_YMin extends ScalarFunction {
         @DataTypeHint("Double")
         public Double eval(@DataTypeHint(value = "RAW", bridgedTo = org.locationtech.jts.geom.Geometry.class) Object o){
@@ -593,6 +609,12 @@ public class Functions {
             Geometry geom = (Geometry) o;
             return org.apache.sedona.common.Functions.isValid(geom);
         }
+
+        @DataTypeHint("Boolean")
+        public Boolean eval(@DataTypeHint(value = "RAW", bridgedTo = org.locationtech.jts.geom.Geometry.class) Object o, @DataTypeHint("Integer") Integer flag) {
+            Geometry geom = (Geometry) o;
+            return org.apache.sedona.common.Functions.isValid(geom, flag);
+        }
     }
 
     public static class ST_Normalize extends ScalarFunction {
@@ -1070,4 +1092,18 @@ public class Functions {
 
         }
     }
+
+    public static class ST_IsValidReason extends ScalarFunction {
+        @DataTypeHint("String")
+        public String eval(@DataTypeHint(value = "RAW", bridgedTo = org.locationtech.jts.geom.Geometry.class) Object o) {
+            Geometry geom = (Geometry) o;
+            return org.apache.sedona.common.Functions.isValidReason(geom);
+        }
+
+        @DataTypeHint("String")
+        public String eval(@DataTypeHint(value = "RAW", bridgedTo = org.locationtech.jts.geom.Geometry.class) Object o, @DataTypeHint("Integer") Integer flag) {
+            Geometry geom = (Geometry) o;
+            return org.apache.sedona.common.Functions.isValidReason(geom, flag);
+        }
+    }
 }
