@@ -80,10 +80,10 @@ SEDONA_LISTED_SQL_FUNCTIONS = [
      .with_transform("ST_ASText")),
     (SuiteContainer.empty()
      .with_function_name("ST_Transform")
-     .with_arguments(["ST_GeomFromText('POINT(21.5 52.5)')", "'epsg:4326'", "'epsg:2180'"])
+     .with_arguments(["ST_GeomFromText('POINT(52.5 21.5)')", "'epsg:4326'", "'epsg:2180'"])
      .with_expected_result(-2501415.806893427)
      #.with_expected_result("POINT (-2501415.806893427 4119952.52325666)")
-     .with_transform("ST_X")),
+     .with_transform("ST_Y")),
      #.with_transform("ST_ASText")),
     (SuiteContainer.empty()
      .with_function_name("ST_Intersection")
@@ -100,7 +100,7 @@ SEDONA_LISTED_SQL_FUNCTIONS = [
      .with_expected_result(1.0)
      .with_transform("ST_AREA")),
     (SuiteContainer.empty()
-     .with_function_name("ST_PrecisionReduce")
+     .with_function_name("ST_ReducePrecision")
      .with_arguments(["ST_GeomFromText('POLYGON ((21 53, 22 53, 22 52, 21 52, 21 53))')", "9"])
      .with_expected_result(1.0)
      .with_transform("ST_AREA")),
@@ -267,13 +267,8 @@ SEDONA_LISTED_SQL_FUNCTIONS = [
     (SuiteContainer.empty()
      .with_function_name("ST_MinimumBoundingCircle")
      .with_arguments([
-        "ST_GeomFromText('POLYGON((1 1,0 0, -1 1, 1 1))')"
-    ])
-     .with_expected_result(3.121445152258052)
-     .with_transform("ST_AREA")),
-    (SuiteContainer.empty()
-     .with_function_name("ST_MinimumBoundingCircle")
-     .with_arguments(["ST_GeomFromText('POLYGON((1 1,0 0, -1 1, 1 1))')"])
+        "ST_GeomFromText('POLYGON((1 1,0 0, -1 1, 1 1))')",
+        "8"])
      .with_expected_result(3.121445152258052)
      .with_transform("ST_AREA")),
     (SuiteContainer.empty()
