@@ -22,13 +22,11 @@ import org.apache.spark.sql.catalyst.InternalRow
 import org.apache.spark.sql.catalyst.expressions.{Expression, ImplicitCastInputTypes}
 import org.apache.spark.sql.catalyst.expressions.codegen.CodegenFallback
 import org.apache.spark.sql.catalyst.util.ArrayData
-import org.apache.spark.sql.sedona_sql.UDT.{GeometryUDT, RasterUDT}
+import org.apache.spark.sql.sedona_sql.UDT.GeometryUDT
 import org.apache.spark.sql.types.{AbstractDataType, BinaryType, BooleanType, DataType, DataTypes, DoubleType, IntegerType, LongType, StringType}
 import org.apache.spark.unsafe.types.UTF8String
 import org.locationtech.jts.geom.Geometry
 import org.apache.spark.sql.sedona_sql.expressions.implicits._
-import org.apache.spark.sql.sedona_sql.expressions.raster.implicits._
-import org.geotools.coverage.grid.GridCoverage2D
 
 import scala.collection.convert.ImplicitConversions.`collection AsScalaIterable`
 import scala.reflect.runtime.universe.TypeTag
@@ -75,14 +73,10 @@ abstract class InferredExpression(fSeq: InferrableFunction *)
 // This is a compile time type shield for the types we are able to infer. Anything
 // other than these types will cause a compilation error. This is the Scala
 // 2 way of making a union type.
-sealed class InferrableType[T: TypeTag]
+class InferrableType[T: TypeTag]
 object InferrableType {
   implicit val geometryInstance: InferrableType[Geometry] =
     new InferrableType[Geometry] {}
-  implicit val gridCoverage2DInstance: InferrableType[GridCoverage2D] =
-    new InferrableType[GridCoverage2D] {}
-  implicit val gridCoverage2DArrayInstance: InferrableType[Array[GridCoverage2D]] =
-    new InferrableType[Array[GridCoverage2D]] {}
   implicit val geometryArrayInstance: InferrableType[Array[Geometry]] =
     new InferrableType[Array[Geometry]] {}
   implicit val javaDoubleInstance: InferrableType[java.lang.Double] =
@@ -127,8 +121,8 @@ object InferredTypes {
       expr => input => expr.toGeometry(input)
     } else if (t =:= typeOf[Array[Geometry]]) {
       expr => input => expr.toGeometryArray(input)
-    } else if (t =:= typeOf[GridCoverage2D]) {
-      expr => input => expr.toRaster(input)
+    } else if (InferredRasterExpression.isRasterType(t)) {
+      InferredRasterExpression.rasterExtractor
     } else if (t =:= typeOf[Array[Double]]) {
       expr => input => expr.eval(input).asInstanceOf[ArrayData].toDoubleArray()
     } else if (t =:= typeOf[String]) {
@@ -156,14 +150,8 @@ object InferredTypes {
         } else {
           null
         }
-    } else if (t =:= typeOf[GridCoverage2D]) {
-      output => {
-        if (output != null) {
-          output.asInstanceOf[GridCoverage2D].serialize
-        } else {
-          null
-        }
-      }
+    } else if (InferredRasterExpression.isRasterType(t)) {
+      InferredRasterExpression.rasterSerializer
     } else if (t =:= typeOf[String]) {
       output =>
         if (output != null) {
@@ -194,19 +182,8 @@ object InferredTypes {
         } else {
           null
         }
-    } else if (t =:= typeOf[Array[GridCoverage2D]]) {
-      output =>
-        if (output != null) {
-          val rasters = output.asInstanceOf[Array[GridCoverage2D]]
-          val serialized = rasters.map { raster =>
-            val serialized = raster.serialize
-            raster.dispose(true)
-            serialized
-          }
-          ArrayData.toArrayData(serialized)
-        } else {
-          null
-        }
+    } else if (InferredRasterExpression.isRasterArrayType(t)) {
+      InferredRasterExpression.rasterArraySerializer
     } else if (t =:= typeOf[Option[Boolean]]) {
       output =>
         if (output != null) {
@@ -224,10 +201,10 @@ object InferredTypes {
       GeometryUDT
     } else if (t =:= typeOf[Array[Geometry]] || t =:= typeOf[java.util.List[Geometry]]) {
       DataTypes.createArrayType(GeometryUDT)
-    } else if (t =:= typeOf[GridCoverage2D]) {
-      RasterUDT
-    } else if (t =:= typeOf[Array[GridCoverage2D]]) {
-      DataTypes.createArrayType(RasterUDT)
+    } else if (InferredRasterExpression.isRasterType(t)) {
+      InferredRasterExpression.rasterUDT
+    } else if (InferredRasterExpression.isRasterArrayType(t)) {
+      InferredRasterExpression.rasterUDTArray
     } else if (t =:= typeOf[java.lang.Double]) {
       DoubleType
     } else if (t =:= typeOf[java.lang.Integer]) {
