@@ -25,6 +25,7 @@ import org.apache.spark.sql.catalyst.expressions.Expression
 import org.apache.spark.sql.catalyst.util.ArrayData
 import org.apache.spark.sql.sedona_sql.UDT.RasterUDT
 import org.apache.spark.sql.sedona_sql.expressions.InferrableFunctionConverter._
+import org.apache.spark.sql.sedona_sql.expressions.InferrableRasterTypes._
 import org.apache.spark.sql.sedona_sql.expressions.raster.implicits.RasterInputExpressionEnhancer
 import org.apache.spark.sql.sedona_sql.expressions.InferredExpression
 import org.geotools.coverage.grid.GridCoverage2D
@@ -44,7 +45,9 @@ case class RS_Count(inputExpressions: Seq[Expression]) extends InferredExpressio
   }
 
 case class RS_ZonalStats(inputExpressions: Seq[Expression]) extends InferredExpression(
-  inferrableFunction5(RasterBandAccessors.getZonalStats), inferrableFunction4(RasterBandAccessors.getZonalStats),
+  inferrableFunction6(RasterBandAccessors.getZonalStats),
+  inferrableFunction5(RasterBandAccessors.getZonalStats),
+  inferrableFunction4(RasterBandAccessors.getZonalStats),
   inferrableFunction3(RasterBandAccessors.getZonalStats)
 ) {
   protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
@@ -54,7 +57,7 @@ case class RS_ZonalStats(inputExpressions: Seq[Expression]) extends InferredExpr
 
 case class RS_ZonalStatsAll(inputExpressions: Seq[Expression]) extends InferredExpression(
   inferrableFunction2(RasterBandAccessors.getZonalStatsAll), inferrableFunction4(RasterBandAccessors.getZonalStatsAll),
-  inferrableFunction3(RasterBandAccessors.getZonalStatsAll)
+  inferrableFunction3(RasterBandAccessors.getZonalStatsAll), inferrableFunction5(RasterBandAccessors.getZonalStatsAll)
 ) {
   protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
     copy(inputExpressions = newChildren)
