@@ -24,7 +24,7 @@ import org.apache.spark.sql.sedona_sql.expressions.st_aggregates._
 import org.apache.spark.sql.sedona_sql.expressions.st_constructors._
 import org.apache.spark.sql.sedona_sql.expressions.st_functions._
 import org.apache.spark.sql.sedona_sql.expressions.st_predicates._
-import org.junit.Assert.assertEquals
+import org.junit.Assert.{assertEquals, assertTrue}
 import org.locationtech.jts.geom.{Geometry, Polygon}
 import org.locationtech.jts.io.WKTWriter
 import org.locationtech.jts.operation.buffer.BufferParameters
@@ -193,9 +193,27 @@ class dataFrameAPITestScala extends TestBaseScala {
     it("Passed ST_Buffer") {
       val polygonDf = sparkSession.sql("SELECT ST_Point(1.0, 1.0) AS geom")
       val df = polygonDf.select(ST_Buffer("geom", 1.0).as("geom")).selectExpr("ST_ReducePrecision(geom, 2)")
-      val actualResult = df.take(1)(0).get(0).asInstanceOf[Geometry].toText()
-      val expectedResult = "POLYGON ((1.98 0.8, 1.92 0.62, 1.83 0.44, 1.71 0.29, 1.56 0.17, 1.38 0.08, 1.2 0.02, 1 0, 0.8 0.02, 0.62 0.08, 0.44 0.17, 0.29 0.29, 0.17 0.44, 0.08 0.62, 0.02 0.8, 0 1, 0.02 1.2, 0.08 1.38, 0.17 1.56, 0.29 1.71, 0.44 1.83, 0.62 1.92, 0.8 1.98, 1 2, 1.2 1.98, 1.38 1.92, 1.56 1.83, 1.71 1.71, 1.83 1.56, 1.92 1.38, 1.98 1.2, 2 1, 1.98 0.8))"
-      assert(actualResult == expectedResult)
+      var actual = df.take(1)(0).get(0).asInstanceOf[Geometry].toText()
+      var expected = "POLYGON ((1.98 0.8, 1.92 0.62, 1.83 0.44, 1.71 0.29, 1.56 0.17, 1.38 0.08, 1.2 0.02, 1 0, 0.8 0.02, 0.62 0.08, 0.44 0.17, 0.29 0.29, 0.17 0.44, 0.08 0.62, 0.02 0.8, 0 1, 0.02 1.2, 0.08 1.38, 0.17 1.56, 0.29 1.71, 0.44 1.83, 0.62 1.92, 0.8 1.98, 1 2, 1.2 1.98, 1.38 1.92, 1.56 1.83, 1.71 1.71, 1.83 1.56, 1.92 1.38, 1.98 1.2, 2 1, 1.98 0.8))"
+      assertEquals(expected, actual)
+
+      var linestringDf = sparkSession.sql("SELECT ST_GeomFromWKT('LINESTRING(0 0, 50 70, 100 100)') AS geom, 'side=left' as params")
+      var dfLine = linestringDf.select(ST_Buffer("geom", 10, "params").as("geom")).selectExpr("ST_ReducePrecision(geom, 2)")
+      actual = dfLine.take(1)(0).get(0).asInstanceOf[Geometry].toText()
+      expected = "POLYGON ((50 70, 0 0, -8.14 5.81, 41.86 75.81, 43.22 77.35, 44.86 78.57, 94.86 108.57, 100 100, 50 70))"
+      assertEquals(expected, actual)
+
+      linestringDf = sparkSession.sql("SELECT ST_GeomFromWKT('LINESTRING(0 0, 50 70, 70 -3)') AS geom, 'endcap=square' AS params")
+      dfLine = linestringDf.select(ST_Buffer("geom", 10, "params").as("geom")).selectExpr("ST_ReducePrecision(geom, 2)")
+      actual = dfLine.take(1)(0).get(0).asInstanceOf[Geometry].toText()
+      expected = "POLYGON ((43.22 77.35, 44.85 78.57, 46.7 79.44, 48.69 79.91, 50.74 79.97, 52.75 79.61, 54.65 78.85, 56.36 77.72, 57.79 76.27, 58.91 74.55, 59.64 72.64, 79.64 -0.36, 82.29 -10, 63 -15.29, 45.91 47.07, 8.14 -5.81, 2.32 -13.95, -13.95 -2.32, 41.86 75.81, 43.22 77.35))"
+      assertEquals(expected, actual)
+
+      val pointDf = sparkSession.sql("SELECT ST_Point(100, 90) AS geom, 'quad_segs=4' as params")
+      val dfPoint = pointDf.select(ST_Buffer("geom", 200, "params").as("geom")).selectExpr("ST_ReducePrecision(geom, 2)")
+      actual = dfPoint.take(1)(0).get(0).asInstanceOf[Geometry].toText()
+      expected = "POLYGON ((284.78 13.46, 241.42 -51.42, 176.54 -94.78, 100 -110, 23.46 -94.78, -41.42 -51.42, -84.78 13.46, -100 90, -84.78 166.54, -41.42 231.42, 23.46 274.78, 100 290, 176.54 274.78, 241.42 231.42, 284.78 166.54, 300 90, 284.78 13.46))"
+      assertEquals(expected, actual)
     }
 
     it("Passed ST_Envelope") {
@@ -271,11 +289,19 @@ class dataFrameAPITestScala extends TestBaseScala {
     }
 
     it("Passed ST_Transform") {
-      val pointDf = sparkSession.sql("SELECT ST_Point(1.0, 1.0) AS geom")
-      val df = pointDf.select(ST_Transform($"geom", lit("EPSG:4326"), lit("EPSG:32649")).as("geom")).selectExpr("ST_ReducePrecision(geom, 2)")
+      var pointDf = sparkSession.sql("SELECT ST_Point(1.0, 1.0) AS geom")
+      var df = pointDf.select(ST_Transform($"geom", lit("EPSG:4326"), lit("EPSG:32649")).as("geom")).selectExpr("ST_ReducePrecision(geom, 2)")
       val actualResult = df.take(1)(0).get(0).asInstanceOf[Geometry].toText()
       val expectedResult = "POINT (-33741810.95 1823994.03)"
-      assert(actualResult == expectedResult)
+      assertEquals(expectedResult, actualResult)
+
+      pointDf = sparkSession.sql("SELECT ST_Point(40.0, 100.0) AS geom")
+                            .select(ST_SetSRID("geom", 4326).as("geom"))
+      df = pointDf.select(ST_Transform($"geom", lit("EPSG:32649")).as("geom"))
+                  .selectExpr("ST_ReducePrecision(geom, 2)")
+      val actual = df.take(1)(0).get(0).asInstanceOf[Geometry].toText()
+      val expected = "POINT (1560393.11 10364606.84)"
+      assertEquals(expected, actual)
     }
 
     it("Passed ST_Intersection") {
@@ -291,6 +317,20 @@ class dataFrameAPITestScala extends TestBaseScala {
       val df = polygonDf.select(ST_IsValid("geom"))
       val actualResult = df.take(1)(0).get(0).asInstanceOf[Boolean]
       assert(actualResult)
+
+      // Geometry that is invalid under both OGC and ESRI standards
+      val selfTouchingWKT = "POLYGON ((0 0, 2 0, 1 1, 2 2, 0 2, 1 1, 0 0))"
+      val specialCaseTable = Seq(selfTouchingWKT).toDF("wkt").select(ST_GeomFromWKT($"wkt").as("geom"))
+
+      // Test with OGC flag (OGC_SFS_VALIDITY = 0)
+      val ogcValidityTable = specialCaseTable.select(ST_IsValid($"geom", lit(0)))
+      val ogcValidity = ogcValidityTable.take(1)(0).getBoolean(0)
+      assertEquals(false, ogcValidity)
+
+      // Test with ESRI flag (ESRI_VALIDITY = 1)
+      val esriValidityTable = specialCaseTable.select(ST_IsValid($"geom", lit(1)))
+      val esriValidity = esriValidityTable.take(1)(0).getBoolean(0)
+      assertEquals(false, esriValidity)
     }
 
     it("Passed ST_ReducePrecision") {
@@ -314,6 +354,10 @@ class dataFrameAPITestScala extends TestBaseScala {
       val actualResult = df.take(1)(0).get(0).asInstanceOf[Geometry].toText()
       val expectedResult = "LINESTRING (0 0, 1 1)"
       assert(actualResult == expectedResult)
+
+      val df2 = sparkSession.sql("SELECT ST_MakeLine(ARRAY(ST_Point(0, 0), ST_Point(1, 1), ST_Point(2, 2)))")
+      val actualResult2 = df2.take(1)(0).get(0).asInstanceOf[Geometry].toText()
+      assert(actualResult2 == "LINESTRING (0 0, 1 1, 2 2)")
     }
 
     it("Passed ST_MakeValid On Invalid Polygon") {
@@ -702,6 +746,14 @@ class dataFrameAPITestScala extends TestBaseScala {
       assert(actualResult == expectedResult)
     }
 
+    it("Passed ST_LineLocatePoint") {
+      val baseDf = sparkSession.sql("SELECT ST_GeomFromWKT('LINESTRING (0 0, 1 1, 2 2)') AS line, ST_GeomFromWKT('POINT (0 2)') AS point")
+      val df = baseDf.select(ST_LineLocatePoint("line", "point"))
+      val actualResult = df.take(1)(0).get(0).asInstanceOf[Double]
+      val expectedResult = 0.5
+      assert(actualResult == expectedResult)
+    }
+
     it ("Passed ST_Multi"){
       val baseDf = sparkSession.sql("SELECT ST_Point(0.0, 0.0) AS point")
       val df = baseDf.select(ST_Multi("point"))
@@ -1213,5 +1265,35 @@ class dataFrameAPITestScala extends TestBaseScala {
       val actual = df.take(1)(0).get(0).asInstanceOf[String]
       assert(expected == actual)
     }
+
+    it("Passed ST_DWithin") {
+      val pointDf = sparkSession.sql("SELECT ST_GeomFromWKT('POINT (0 0)') as origin, ST_GeomFromWKT('POINT (1 0)') as point")
+      val df = pointDf.select(ST_DWithin("origin", "point", 2.0))
+      val actual = df.head()(0).asInstanceOf[Boolean]
+      assertTrue(actual)
+    }
+
+    it("Passed ST_IsValidReason") {
+      // Valid Geometry
+      val validPolygonWKT = "POLYGON ((0 0, 2 0, 2 2, 0 2, 1 1, 0 0))"
+      val validTable = Seq(validPolygonWKT).toDF("wkt").select(ST_GeomFromWKT($"wkt").as("geom"))
+      val validityTable = validTable.select(ST_IsValidReason($"geom"))
+      val validityReason = validityTable.take(1)(0).getString(0)
+      assertEquals("Valid Geometry", validityReason)
+
+      // Geometry that is invalid under both OGC and ESRI standards, but with different reasons
+      val selfTouchingWKT = "POLYGON ((0 0, 2 0, 1 1, 2 2, 0 2, 1 1, 0 0))"
+      val specialCaseTable = Seq(selfTouchingWKT).toDF("wkt").select(ST_GeomFromWKT($"wkt").as("geom"))
+
+      // Test with OGC flag (OGC_SFS_VALIDITY = 0)
+      val ogcValidityTable = specialCaseTable.select(ST_IsValidReason($"geom", lit(0)))
+      val ogcValidityReason = ogcValidityTable.take(1)(0).getString(0)
+      assertEquals("Ring Self-intersection at or near point (1.0, 1.0, NaN)", ogcValidityReason)
+
+      // Test with ESRI flag (ESRI_VALIDITY = 1)
+      val esriValidityTable = specialCaseTable.select(ST_IsValidReason($"geom", lit(1)))
+      val esriValidityReason = esriValidityTable.take(1)(0).getString(0)
+      assertEquals("Interior is disconnected at or near point (1.0, 1.0, NaN)", esriValidityReason)
+    }
   }
 }
