@@ -14,10 +14,11 @@
 package org.apache.spark.sql.execution.datasources.v2.geoparquet.metadata
 
 import org.apache.hadoop.conf.Configuration
+import org.apache.hadoop.fs.Path
 import org.apache.parquet.hadoop.ParquetFileReader
 import org.apache.parquet.hadoop.util.HadoopInputFile
 import org.apache.spark.broadcast.Broadcast
-import org.apache.spark.sql.catalyst.{FileSourceOptions, InternalRow}
+import org.apache.spark.sql.catalyst.InternalRow
 import org.apache.spark.sql.catalyst.expressions.GenericInternalRow
 import org.apache.spark.sql.catalyst.util.{ArrayBasedMapData, GenericArrayData}
 import org.apache.spark.sql.connector.read.PartitionReader
@@ -37,7 +38,6 @@ case class GeoParquetMetadataPartitionReaderFactory(
     dataSchema: StructType,
     readDataSchema: StructType,
     partitionSchema: StructType,
-    options: FileSourceOptions,
     filters: Seq[Filter]) extends FilePartitionReaderFactory {
 
   override def buildReader(partitionedFile: PartitionedFile): PartitionReader[InternalRow] = {
@@ -55,9 +55,9 @@ object GeoParquetMetadataPartitionReaderFactory {
   private def readFile(configuration: Configuration,
                partitionedFile: PartitionedFile,
                readDataSchema: StructType): Iterator[InternalRow] = {
-    val filePath = partitionedFile.toPath.toString
+    val filePath = partitionedFile.filePath
     val metadata = ParquetFileReader.open(
-      HadoopInputFile.fromPath(partitionedFile.toPath, configuration))
+      HadoopInputFile.fromPath(new Path(filePath), configuration))
       .getFooter.getFileMetaData.getKeyValueMetaData
     val row = GeoParquetMetaData.parseKeyValueMetaData(metadata) match {
       case Some(geo) =>
