@@ -25,9 +25,11 @@ import org.apache.parquet.schema.{GroupType, OriginalType, Type}
 import org.apache.spark.internal.Logging
 import org.apache.spark.sql.catalyst.InternalRow
 import org.apache.spark.sql.catalyst.expressions._
+import org.apache.spark.sql.catalyst.types.{PhysicalByteType, PhysicalShortType}
+import org.apache.spark.sql.catalyst.util.RebaseDateTime.RebaseSpec
 import org.apache.spark.sql.catalyst.util.{ArrayBasedMapData, CaseInsensitiveMap, DateTimeUtils, GenericArrayData}
+import org.apache.spark.sql.execution.datasources.DataSourceUtils
 import org.apache.spark.sql.internal.SQLConf
-import org.apache.spark.sql.internal.SQLConf.LegacyBehaviorPolicy
 import org.apache.spark.sql.sedona_sql.UDT.GeometryUDT
 import org.apache.spark.sql.types._
 import org.apache.spark.unsafe.types.UTF8String
@@ -72,10 +74,10 @@ import scala.collection.mutable.ArrayBuffer
  * @param catalystType Spark SQL schema that corresponds to the Parquet record type. User-defined
  *        types other than [[GeometryUDT]] should have been expanded.
  * @param convertTz the optional time zone to convert to int96 data
- * @param datetimeRebaseMode the mode of rebasing date/timestamp from Julian to Proleptic Gregorian
- *                           calendar
- * @param int96RebaseMode the mode of rebasing INT96 timestamp from Julian to Proleptic Gregorian
- *                           calendar
+ * @param datetimeRebaseSpec the specification of rebasing date/timestamp from Julian to Proleptic
+ *                           Gregorian calendar: mode + optional original time zone
+ * @param int96RebaseSpec the specification of rebasing INT96 timestamp from Julian to Proleptic
+ *                        Gregorian calendar
  * @param parameters Options for reading GeoParquet files. For example, if legacyMode is enabled or not.
  * @param updater An updater which propagates converted field values to the parent container
  */
@@ -84,8 +86,8 @@ private[parquet] class GeoParquetRowConverter(
                                             parquetType: GroupType,
                                             catalystType: StructType,
                                             convertTz: Option[ZoneId],
-                                            datetimeRebaseMode: LegacyBehaviorPolicy.Value,
-                                            int96RebaseMode: LegacyBehaviorPolicy.Value,
+                                            datetimeRebaseSpec: RebaseSpec,
+                                            int96RebaseSpec: RebaseSpec,
                                             parameters: Map[String, String],
                                             updater: ParentContainerUpdater)
   extends ParquetGroupConverter(updater) with Logging {
@@ -137,14 +139,14 @@ private[parquet] class GeoParquetRowConverter(
    */
   def currentRecord: InternalRow = currentRow
 
-  private val dateRebaseFunc = GeoDataSourceUtils.creteDateRebaseFuncInRead(
-    datetimeRebaseMode, "Parquet")
+  private val dateRebaseFunc = DataSourceUtils.createDateRebaseFuncInRead(
+    datetimeRebaseSpec.mode, "Parquet")
 
-  private val timestampRebaseFunc = GeoDataSourceUtils.creteTimestampRebaseFuncInRead(
-    datetimeRebaseMode, "Parquet")
+  private val timestampRebaseFunc = DataSourceUtils.createTimestampRebaseFuncInRead(
+    datetimeRebaseSpec, "Parquet")
 
-  private val int96RebaseFunc = GeoDataSourceUtils.creteTimestampRebaseFuncInRead(
-    int96RebaseMode, "Parquet INT96")
+  private val int96RebaseFunc = DataSourceUtils.createTimestampRebaseFuncInRead(
+    int96RebaseSpec, "Parquet INT96")
 
   // Converters for each field.
   private[this] val fieldConverters: Array[Converter with HasParentContainerUpdater] = {
@@ -233,7 +235,7 @@ private[parquet] class GeoParquetRowConverter(
       case ByteType =>
         new ParquetPrimitiveConverter(updater) {
           override def addInt(value: Int): Unit =
-            updater.setByte(value.asInstanceOf[ByteType#InternalType])
+            updater.setByte(value.asInstanceOf[PhysicalByteType#InternalType])
 
           override def addBinary(value: Binary): Unit = {
             val bytes = value.getBytes
@@ -246,7 +248,7 @@ private[parquet] class GeoParquetRowConverter(
       case ShortType =>
         new ParquetPrimitiveConverter(updater) {
           override def addInt(value: Int): Unit =
-            updater.setShort(value.asInstanceOf[ShortType#InternalType])
+            updater.setShort(value.asInstanceOf[PhysicalShortType#InternalType])
         }
 
       // For INT32 backed decimals
@@ -282,7 +284,7 @@ private[parquet] class GeoParquetRowConverter(
       case TimestampType if parquetType.getOriginalType == OriginalType.TIMESTAMP_MILLIS =>
         new ParquetPrimitiveConverter(updater) {
           override def addLong(value: Long): Unit = {
-            val micros = GeoDateTimeUtils.millisToMicros(value)
+            val micros = DateTimeUtils.millisToMicros(value)
             updater.setLong(timestampRebaseFunc(micros))
           }
         }
@@ -355,8 +357,8 @@ private[parquet] class GeoParquetRowConverter(
           parquetType.asGroupType(),
           t,
           convertTz,
-          datetimeRebaseMode,
-          int96RebaseMode,
+          datetimeRebaseSpec,
+          int96RebaseSpec,
           parameters,
           wrappedUpdater)
 
